"""
    parse_onda("name-of-calibration-file.txt")

Translates calibration text files from Onda
into the format required by this package.
Output is a .jl file written to the current active directory.
Running the new file will load the calibration data to be used by HydrophoneCalibrations.

"""
function parse_onda(filename)
    info = Dict()
    read_file_as_strings!(info, filename)
    parse_header!(info)
    parse_data!(info)
    get_raw_header!(info)
    create_device_name!(info)
    savepath = write_to_file!(info)
    return savepath
end

function read_file_as_strings!(info, filename)
    data_in = readlines(filename)
    if length(data_in) == 1
        data_in = split(data[1], '\r')
    end
    info["data_in"] = data_in
    return nothing
end

function parse_header!(info)
    data_in = info["data_in"]
    nlines = 0
    for line in data_in
        nlines += 1
        store_field(info, "MFG", line)
        store_field(info, "MODEL", line)
        store_field(info, "SN", line)
        store_field(info, "AMP_GAIN_NOM_DB", line)
        store_field(info, "FILE_NAME", line)
        store_field(info, "AMP_CAL_DATE", line)
        store_field(info, "HYD_APERTURE_NOM_UM", line)
        store_field(info, "Calibration_DATE", line)

        if is_header_end(line)
            break
        end
    end
    info["n_header_lines"] = nlines
    return nothing
end

function store_field(info, field, line)
    if found(field, line)
        columns = split(line)
        if length(columns) == 2
            info[field] = columns[2]
            println("$field: $(columns[2])")
        elseif length(columns) == 1
            println("$field: $(columns)")
            info[field] = columns
        else
            error("Could not parse line. Too many elements in:\n \"$columns\"")
        end
    end
    return nothing
end

function found(field, line)
    return occursin(field, line) && !occursin("NONE", line)
end

is_header_end(line) = occursin("HEADER_END", line)
function parse_data!(info)
    nlines = info["n_header_lines"]
    info["data"] = info["data_in"][nlines + 1:end]
    return nothing
end

function get_raw_header!(info)
    data = info["data_in"]
    header = String[]
    for line in data
        if is_header_end(line)
            break;
        else
            push!(header, line)
        end
    end
    info["HEADER"] = header
    return nothing
end


function create_device_name!(info)
    if is_preamp(info)
        device_name = create_preamp_name(info)
    else
        device_name = create_hydrophone_name(info)
    end
    device_name = replace(device_name, "-"=>"_")
    device_name = replace(device_name, "__"=>"_")
    info["device_name"] = device_name
    return nothing
end

is_preamp(info) = haskey(info, "AMP_GAIN_NOM_DB")

function create_preamp_name(info)
    model = info["MODEL"]
    sn = info["SN"]
    date = info["AMP_CAL_DATE"]
    db = parse(Int, info["AMP_GAIN_NOM_DB"])
    return "Onda_$(model)_$(sn)_$(db)dB___$(date)"
end

function create_hydrophone_name(info)
    model = info["MODEL"]
    sn = info["SN"]
    width = info["HYD_APERTURE_NOM_UM"]
    date = info["Calibration_DATE"]
    return "Onda_$(model)_$(width)_$(sn)___$(date)"
end

function write_to_file!(info)
    device_name = info["device_name"]
    savepath = "$(device_name).jl"
    open(savepath, "w") do io
        write_header!(io, info)
        println(io, "@calibration :Onda_$(device_name) [")
        for line in info["data"]
            println(io, line)
        end
        println(io, "]")
    end
    return savepath
end


function write_header!(io, info)
    println(io, "\"\"\"")
    for line in info["HEADER"]
        println(io, "$line")
    end
    println(io, "\"\"\"")
    println(io, "\n\n")
    return nothing
end
