"""
    volt_to_pressure(15e6; hydrophone_id = :Onda_HGL0200_2322, preamp_id = :Onda_AH2020_1238_20dB)
    volt_to_pressure(15e6; hydrophone_id = :PA_3197)

# Arguments
- `f0`: Frequency in Hz.
- `hydrophone_id`: Currently accepts two: :Onda_HGL0200_2322, :PA_3197.
- `preamp_id`: Only for :Onda_HGL0200_2322. Choices are: :Onda_AH2020_1238_0dB, :Onda_AH2020_1238_20dB.
# Returns
- `factor`: conversion factor in form of pascal/volt.
"""
function volt_to_pressure(f0, hydrophone_id::Symbol)
    f0 = enforce_hertz_unit(f0)
    calibration_data, _ = get_calibration(hydrophone_id)
    frequencies = calibration_data[:, 1] * u"MHz"

    if occursin("PA_", String(hydrophone_id))
        volt_per_pascal = calibration_data[:, 2] * u"mV/MPa"
    else
        volt_per_pascal = calibration_data[:, 2] * u"dBV/μPa"
    end

    volt_per_pascal = interp(frequencies, volt_per_pascal, f0)
    return calculate_pascal_per_volt(volt_per_pascal)
end

function calculate_pascal_per_volt(volt_per_pascal)
    volt_per_pascal = uconvertrp(u"V/Pa", volt_per_pascal)
    pascal_per_volt = 1/volt_per_pascal
    return pascal_per_volt
end


function volt_to_pressure(f0, hydrophone_id::Symbol, preamp_id::Nothing)
    return volt_to_pressure(f0, hydrophone_id)
end

function volt_to_pressure(f0, hydrophone_id::Symbol, preamp_id::Symbol)
    factor, _ = volt_to_pressure_and_phase(f0, hydrophone_id, preamp_id)
    return factor
end

function volt_to_pressure_and_phase(f0, hydrophone_id::Symbol, preamp_id::Symbol)
    f0 = enforce_hertz_unit(f0)
    calibration_data, preamp_data = get_calibration(hydrophone_id, preamp_id)

    hydrophone_freqs = calibration_data[:, 1] * u"MHz"
    hydrophone_volt_per_pascal = calibration_data[:, 2] * u"dBV/μPa"
    hydrophone_capacitances = calibration_data[:, 5] * u"pF"
    hydrophone_volt_per_pascal_at_f0 = interp(hydrophone_freqs, hydrophone_volt_per_pascal, f0)
    hydrophone_capacitance_at_f0 = interp(hydrophone_freqs, hydrophone_capacitances, f0)

    preamp_freqs = preamp_data[:,1] * u"MHz"
    preamp_gains = uconvertrp.(NoUnits, preamp_data[:, 2] * u"dB")
    preamp_phases = preamp_data[:, 3]
    preamp_capacitances = preamp_data[:, 4] * u"pF"
    preamp_gain_at_f0 = interp(preamp_freqs, preamp_gains, f0)
    preamp_phase_at_f0 = interp(preamp_freqs, preamp_phases, f0)
    preamp_capacitance_at_f0 = interp(preamp_freqs, preamp_capacitances, f0)

    voltage_factor = hydrophone_capacitance_at_f0 ./ (hydrophone_capacitance_at_f0 + preamp_capacitance_at_f0)

    # Combine result
    combined_volt_per_pascal = preamp_gain_at_f0 * hydrophone_volt_per_pascal_at_f0 * voltage_factor

    return calculate_pascal_per_volt(combined_volt_per_pascal), preamp_phase_at_f0
end


function interp(frequencies, sensitivities, f0)
    f_lower, f_upper = frequencies[1], frequencies[end]
    if f0 < f_lower || f0 > f_upper
        throw("f0 must be between $f_lower-$f_upper")
    end

    interpolant = Interpolations.linear_interpolation(frequencies, sensitivities)
    interpolated_value = interpolant(f0)
    return interpolated_value
end


function enforce_hertz_unit(center_frequency)
    if isa(center_frequency, Unitful.Frequency)
        center_frequency = uconvert(u"Hz", center_frequency)
    elseif isa(center_frequency, Real)
        center_frequency = center_frequency*u"Hz"
    else
        error("Center frequency must have a frequency unit or be a Real number.")
    end

    return center_frequency
end
